<?php
   /**
    * This class contains a wide range of member functions designed to make the life of a programmer easier
    */
   class onWord_helper {
      private $plugin_folder;

      public function __construct($plugin_folder = "") {
         if (empty($plugin_folder)) {
				$plugin_path = dirname(dirname(__DIR__));
				$pieces = explode(DIRECTORY_SEPARATOR, $plugin_path);
				$plugin_folder = array_pop($pieces);
			}
         $this->plugin_folder = $plugin_folder;
      }



      /**
       * Calls WordPress's current_user_can on the capability
       * Works in an AJAX context!
       *
       */
      public function authenticate($capability) {
         //load wordpress if need be
         if (!is_callable("current_user_can")) {
            $site_dir = $this->site_dir();
            require_once ("$site_dir/wp-blog-header.php");
            require_once ("$site_dir/wp-includes/capabilities.php");
         }
         return current_user_can($capability);
      }



      /**
       * Deletes the given file
       */
      public function delete_file($class, $id, $filename) {
         $site_dir = $this->site_dir();
         return unlink("$site_dir/wp-content/uploads/$class/$id/$filename");
      }






      /**
       * Returns an href-suitable string with the file's url
       */
      public function file_url($class, $id, $filename) {
         $site_url = $this->site_url();
         return "$site_url/wp-content/uploads/$class/$id/$filename";
      }






      /**
       * Creates the html for a form, in a table layout
       *
       * $name is the name of the form
       * $action is the page that will process the form
       * $fields expects an array in the form:
       * array(label => array("type"=>type, "name"=>name, "value"=>value), label=>...)
       * Where:
       * label is the value that will appear in the left column
       * type is the name of the specific input_*() function you would call to render this input,
		 *		for example: "input_hidden" or "input_text"
       * name is the name of the input, as passed to this function
       * value is the value you would pass to this function
       *
       * At this time, only default values are used for show_time, year_range_bottom, and year_range_top
       *
       * Remember that a file input expects a very specific format for its $value
		 *
		 * You may use the "name" field on input_recaptcha types to specify your recaptcha public key
		 * Don't forget to change the "name" to your recaptcha private key when you post this form
       *
       */
      public function form($name, $action, $fields, $attributes = "enctype='multipart/form-data'", $submit = "Submit") {
         $output = "<form name='$name' id='$name' action='$action' method='POST' $attributes>";
         $output .= "<table>";
         foreach ($fields as $label=>$field) {
				if ($field["type"] == "input_hidden") {
					$output .= $this->$field["type"]($field["name"], $field["value"]);
				}
				else {
					$output .= "<tr>";
					$output .= "<td>";
					$output .= "<span class='$name-label'>$label</span>";
					$output .= "</td>";
					$output .= "<td>";
					$output .= $this->$field["type"]($field["name"], $field["value"]);
					$output .= "</td>";
					$output .= "</tr>";
				}
         }
         $output .= "<tr>";
         $output .= "<td colspan='2'>";
         $output .= "<input type='submit' name='$submit' value='$submit' />";
         $output .= "</td>";
         $output .= "</tr>";
         $output .= "</table>";
         $output .= "</form>";
         return $output;
      }



      /**
       * A simple helper to get the id from the current page's url
       * Expects URL format as http://www.example.com/my/page/numeric_id_goes_here/
       * Which, conveniently, is how WordPress does pretty permalinks
       */
      public function get_id() {
         $url = explode("/", $_SERVER['REQUEST_URI']);
         return $url[count($url) - 2];
      }



      /**
       * Returns a string that can be output to include the ajax function
       */
      public function include_ajax() {
         $plugin_name = $this->plugin_folder;
         $site_url = $this->site_url();
         $file_url = "$site_url/wp-content/plugins/$plugin_name/ajax/ajax";
         $output = "<script type='text/javascript'>";
         $output .= "ajax_listener = '$file_url.php';";
         $output .= "</script>";
         $output .= "<script type='text/javascript' src='$file_url.js'></script>";
         return $output;
      }




      /**
       * Returns a string that can be output to include a javascript file
       */
      public function include_script($class, $script) {
         $plugin_name = $this->plugin_folder;
         $site_url = $this->site_url();
         return "<script type='text/javascript' src='$site_url/wp-content/plugins/$plugin_name/classes/$class/scripts/$script.js'></script>";
      }



      /**
       * Returns a string that can be output to include a css file
       */
      public function include_style($class, $style) {
         $plugin_name = $this->plugin_folder;
         $site_url = $this->site_url();
         return "<link rel='stylesheet' type='text/css' href='$site_url/wp-content/plugins/$plugin_name/classes/$class/styles/$style.css' />";
      }



      /**
       * Returns a form input as a string.
       * $value is an integer like would be returned from time() or strtotime(), or you can leave value empty to use the current time
       * $show_time is a boolean value that will add a time selector to the calendar, which is on by default
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_calendar($name, $value = "", $show_time = true, $attributes = "") {
         $output = $this->include_script("onWord_helper", "calendar");
         $output .= $this->include_style("onWord_helper", "calendar");
         if (empty($value) || !is_numeric($value)) {
            $value = time();
         }
         if ($show_time) {
            $show = "datetime";
            $time = date("Y-m-d H:i", $value);
         }
         else {
            $show = "date";
            $time = date("Y-m-d", $value);
         }
         $output .= "<input type='text' name='$name' id='$name' onclick='showCalendar(\"$name\", \"$show\");' value='$time' $attributes/>";
         $output .= "<div id ='" . $name . "_modal-background' class='modal-background'><div id='" . $name . "_calendar' class='onWord-calendar'></div></div>";
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is an array of the form:
       * array(
		 *		"values" => array(
		 *			"label"=>"value",
		 *			"label"=>"value"...
		 *			),
		 *		"selected" => array("selected_value", "selected_value"...)
		 *		)
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_checkbox($name, $value = "", $attributes = "") {
         $i = 0;
         $output = "";
         foreach ($value["values"] as $label=>$value_data) {
            $output .= "<input type='checkbox' name='" . $name . "[]' id='$name$i' value='$value_data'";
            if (in_array($value_data, $value["selected"])) {
               $output .= " checked";
            }
            $output .= " $attributes/>&nbsp;&nbsp;$label<br>";
            $i++;
         }
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is an integer like would be returned from time() or strtotime(), or you can leave value empty to use the current time
       * $year_range_bottom is used to determine the lowest year shown, and is used when calculating $min = $current_year - $year_range_bottom
      * $year_range_top is used to determine the highest year shown, and is used when calculating $max = $current_year + $year_range_top
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_date_select($name, $value = "", $year_range_bottom = 10, $year_range_top = 10, $attributes = "") {
         if (empty($value) || !is_numeric($value)) {
            $value = time();
         }
         $month = date("n", $value);
         $day = date("j", $value);
         $year = date("Y", $value);
         $output = "<select name='" . $name . "_month' id='" . $name . "_month' $attributes>";
         for ($i = 1; $i < 13; $i++) {
            $output .= "<option value='$i'";
            if ($month == $i) {
               $output .= " selected";
            }
            $output .= ">" . date("M", strtotime("$i/01/1970")) . "</option>";
         }
         $output .= "</select>/";
         $output .= "<select name='" . $name . "_day' id='" . $name . "_day' $attributes>";
         for ($i = 1; $i < 32; $i++) {
            $output .= "<option value='$i'";
            if ($day == $i) {
               $output .= " selected";
            }
            $output .= ">" . date("j", strtotime("1/$i/1970")) . "</option>";
         }
         $output .= "</select>/";
         $output .= "<select name='" . $name . "_year' id='" . $name . "_year' $attributes>";
         $this_year = date("Y");
         for ($i = $this_year - $year_range_bottom; $i <= $this_year + $year_range_top; $i++) {
            $output .= "<option value='$i'";
            if ($year == $i) {
               $output .= " selected";
            }
            $output .= ">$i</option>";
         }
         $output .= "</select>";
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is an integer like would be returned from time() or strtotime(), or you can leave value empty to use the current time
       * $year_range_bottom is used to determine the lowest year shown, and is used when calculating $min = $current_year - $year_range_bottom
      * $year_range_top is used to determine the highest year shown, and is used when calculating $max = $current_year + $year_range_top
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_datetime_select($name, $value = "", $year_range_bottom = 10, $year_range_top = 10, $attributes = "") {
         $output = $this->input_date_select($name, $value, $year_range_bottom, $year_range_top, $attributes);
         $output .= "&nbsp;&nbsp;";
         $output .= $this->input_time_select($name, $value, $attributes);
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is an integer like would be returned from time() or strtotime(), or you can leave value empty to use the current time
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_time_select($name, $value = "", $attributes = "") {
         if (empty($value) || !is_numeric($value)) {
            $value = time();
         }
         $hour = date("g", $value);
         $minute = date("i", $value);
         $ante = date("A", $value);
         $output = "";
         $output .= "<select name='" . $name . "_hour' id='" . $name . "_hour' $attributes>";
         for ($i = 1; $i < 13; $i++) {
            $output .= "<option value='$i'";
            if ($hour == $i) {
               $output .= " selected";
            }
            $output .= ">$i</option>";
         }
         $output .= "</select>:";
         $output .= "<select name='" . $name . "_minute' id='" . $name . "_minute' $attributes>";
         for ($i = 0; $i < 60; $i++) {
            $min = ($i < 10) ? "0$i" : $i;
            $output .= "<option value='$min'";
            if ($minute == $min) {
               $output .= " selected";
            }
            $output .= ">$min</option>";
         }
         $output .= "</select>:";
         $output .= "<select name='" . $name . "_ante' id='" . $name . "_ante' $attributes>";
         for ($i = 0; $i < 2; $i++) {
            $ant = ($i == 0) ? "AM" : "PM";
            $output .= "<option value='$i'";
            if ($ante == $ant) {
               $output .= " selected";
            }
            $output .= ">$ant</option>";
         }
         $output .= "</select>";
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is an array of the form:
       * array("filename" => filename, "class" => calling class, "id" => object id)
       * this class and id are used to get an existing file's url and will be ignored if $value["filename"] is empty
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_file($name, $value = array(), $attributes = "") {
         $output = "";
         if (!empty($value["filename"])) {
            $output .= "Replace: <a href='";
            $output .= $this->file_url($value["class"], $value["id"], $value["filename"]);
            $output .= "'>" . $value["filename"] . "</a><br>";
         }
         $output .= "<input type='file' name='$name' id='$name' $attributes/>";
         if (!empty($value["filename"])) {
            $output .= "<input type='checkbox' name='remove_$name' value='remove' />&nbsp;&nbsp;Remove<br>";
            $output .= "<input type='hidden' name='existing_$name' value='" . $value["filename"] . "' />";
         }
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is as string
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_hidden($name, $value = "", $attributes = "") {
         return "<input type='hidden' name='$name' id='$name' value='$value' $attributes/>";
      }



      /**
       * Returns a paragraph for use as a form separator.
       * $value is as string
       * $attributes will be included inside the p tag
       *
       */
      public function input_paragraph($name, $value = "", $attributes = "") {
         return "<p name='$name' id='$id' $attributes>$value</p>";
      }



      /**
       * Returns a form input as a string.
       * $value is an array of the form:
       * array(
		 *		"values" => array(
		 *			"label"=>"value",
		 *			"label"=>"value"...
		 *			),
		 *		"selected" => array("selected_value", "selected_value"...)
		 *		)
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_radio($name, $value = "", $attributes = "") {
         $i = 0;
         $output = "";
         foreach ($value["values"] as $label=>$value_data) {
            $output .= "<input type='radio' name='$name' id='$name$i' value='$value_data'";
            if (in_array($value_data, $value["selected"])) {
               $output .= " checked";
            }
            $output .= " $attributes/>&nbsp;&nbsp;$label<br>";
            $i++;
         }
         return $output;
      }



      /**
       * Returns a captcha form input as a string.
       */
      public function input_recaptcha($public_key) {
			$plugin_name = $this->plugin_folder;
         $site_dir = $this->site_dir();
         $file_dir = "$site_dir/wp-content/plugins/$plugin_name/classes/onWord_helper/recaptchalib.php";
         require_once($file_dir);
			return recaptcha_get_html($public_key);
      }



      /**
       * Returns a form input as a string.
       * $value is an array of the form:
       * array(
		 *		"values" => array(
		 *			"label"=>"value",
		 *			"label"=>"value"...
		 *			),
		 *		"selected" => array("selected_value", "selected_value"...)
		 *		)
       * $attributes will be included inside every appropriate input tag
		 * So you could use "multiple='multiple'" as your attributes and get a multiple-select input
       *
       */
      public function input_select($name, $value = "", $attributes = "") {
         $output = "<select name='$name' id='$name' $attributes>";
         foreach ($value["values"] as $label=>$value_data) {
            $output .= "<option value='$value_data'";
            if (in_array($value_data, $value["selected"])) {
               $output .= " selected";
            }
            $output .= ">$label</option>";
         }
         $output .= "</select>";
         return $output;
      }



      /**
       * Returns a form input as a dropdown of US states.
       * $value is a string abbreviation
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_state($name, $value = "", $attributes = "") {
			$states = array('' => "Select a State", 'AL' => "Alabama", 'AK' => "Alaska", 'AZ' => "Arizona", 'AR' => "Arkansas", 'CA' => "California", 'CO' => "Colorado", 'CT' => "Connecticut", 'DE' => "Delaware", 'DC' => "District Of Columbia", 'FL' => "Florida", 'GA' => "Georgia", 'HI' => "Hawaii", 'ID' => "Idaho", 'IL' => "Illinois", 'IN' => "Indiana", 'IA' => "Iowa", 'KS' => "Kansas", 'KY' => "Kentucky", 'LA' => "Louisiana", 'ME' => "Maine", 'MD' => "Maryland", 'MA' => "Massachusetts", 'MI' => "Michigan", 'MN' => "Minnesota", 'MS' => "Mississippi", 'MO' => "Missouri", 'MT' => "Montana", 'NE' => "Nebraska", 'NV' => "Nevada", 'NH' => "New Hampshire", 'NJ' => "New Jersey", 'NM' => "New Mexico", 'NY' => "New York", 'NC' => "North Carolina", 'ND' => "North Dakota", 'OH' => "Ohio", 'OK' => "Oklahoma", 'OR' => "Oregon", 'PA' => "Pennsylvania", 'RI' => "Rhode Island", 'SC' => "South Carolina", 'SD' => "South Dakota", 'TN' => "Tennessee", 'TX' => "Texas", 'UT' => "Utah", 'VT' => "Vermont", 'VA' => "Virginia", 'WA' => "Washington", 'WV' => "West Virginia", 'WI' => "Wisconsin", 'WY' => "Wyoming");
			$output = "<select name='$name' id='$name' $attributes>";
         foreach ($states as $abbr=>$state) {
            $output .= "<option value='$abbr'";
            if ($value == $abbr) {
               $output .= " selected";
            }
            $output .= ">$state</option>";
         }
         $output .= "</select>";
         return $output;
      }



      /**
       * Returns a form input as a string.
       * $value is a string
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_text($name, $value = "", $attributes = "") {
         return "<input type='text' name='$name' id='$name' value='$value' $attributes/>";
      }



      /**
       * Returns a form input as a string.
       * $value is as string
       * $attributes will be included inside every appropriate input tag
       *
       */
      public function input_textarea($name, $value = "", $attributes = "") {
         return "<textarea name='$name' id='$name' $attributes>$value</textarea>";
      }



      /**
       * Displays a given input in a recursively formatted way
       * $value can be either a string or a (potentially nested) array
       * You can use any string for $tag, but any "<" characters will be replaced with "</" when the closing tag is created
       *
       * $sub_tag works the same way as $tag, but $sub_tag will be used if $value is an array, with a sub-array
       *
       * For example, if you wanted to account for nested lists storred in $arr, you would use:
       * $helper = new onWord_helper();
       * echo "<ul>" . $helper->output($arr, "<li>", "class='list-item'", "<ul>", "class='sub-list'") ."</ul>;
       *
       * If you want to do a table, with an array of row arrays you can use:
       * $helper = new onWord_helper();
       * echo "<table>" . $helper->output($arr, "<td>", "class='my-table-cell'", "<tr>", "class='my-table-row'") . "</table>;
       *
       * A table with headers requires a bit more thinking, but it's still much quicker than coding it yourself
       * $helper = new onWord_helper();
       * $output = "<table><tr><th>Header 1</th><th>Header 2</th><th>Header 3</th></tr>";
       * $output .= $helper->output($arr, "<td>", "class='my-table-cell'", "<tr>", "class='my-table-row'") . "</table>;
       */
      public function output($value, $tag = "<p>", $attributes = "", $sub_tag = "<p>", $sub_attributes = "") {
         $closing_tag = str_replace("<", "</", $tag);
         $tag_complete = str_replace(">", " $attributes>", $tag);
         if (!is_array($value)) {
            return $tag_complete . $value . $closing_tag;
         }
         $output = "";
         foreach ($value as $sub_value) {
            if (!is_array($sub_value)) {
               $output .= $tag_complete . $sub_value . $closing_tag;
            }
            else {
               $sub_closing_tag = str_replace("<", "</", $sub_tag);
               $sub_tag_complete = str_replace(">", " $sub_attributes>", $sub_tag);
               $output .= $sub_tag_complete . $this->output($sub_value, $tag, $attributes, $sub_tag, $sub_attributes) . $sub_closing_tag;
            }
         }
         return $output;
      }



      /**
       * Automatically creates "pagination" links for the current page
       * $page_variable is the $_GET variable holding the current page number
       */
public function paginate($num_results, $num_per_page = 30, $page_variable = "page_num", $count_show_pages = 10) {
        $url = $this->site_url() . $_SERVER["REQUEST_URI"];
        if (strpos($url, "&$page_variable=") !== false)
            $url = substr($url, 0, strpos($url, "&$page_variable="));
        if (strpos($url, "?$page_variable=") !== false)
            $url = substr($url, 0, strpos($url, "?$page_variable="));
        if (isset($_SERVER["HTTPS"]) && substr($url, 0, 5) == "http:")
            $url = str_replace("http:", "https:", $url);
        $url .= (strpos($url, "?")) ? "&" : "?";
        $output = "<div class='onWord-pagination'>";
        $pages = ceil($num_results / $num_per_page);
        if ($pages > 1) {
            $left = $_GET[$page_variable] - 1;
            $right = $pages - $_GET[$page_variable];
            if ($left < floor($count_show_pages / 2))
                $start = 1;
            else
                $start = $_GET[$page_variable] - floor($count_show_pages / 2);
            $end = $start + $count_show_pages - 1;
            if ($end > $pages) {
                $start -= ($end - $pages);
                $end = $pages;
                if ($start < 1)
                    $start = 1;
            }
            $output .= "Pages:&nbsp;&nbsp;&nbsp;";
            if ($_GET[$page_variable] != 1) {
                $output .='<a href="' . $url . '" title="First page">&lt;&lt;&lt;</a>&nbsp;&nbsp;&nbsp;';
                $output .='<a href="';
                $output .=($_GET[$page_variable] == 2) ? $url : $url . $page_variable . '=' . ($_GET[$page_variable] - 1);
                $output .='" title="Previous page">&lt;</a>&nbsp;&nbsp;&nbsp;';
            }
            for ($i = $start; $i <= $end; $i++) {
                $output .= "<a href='$url$page_variable=$i'>";
                if ((empty($_GET[$page_variable]) && $i == 1) || (!empty($_GET[$page_variable]) && $i == $_GET[$page_variable])) {
                    $output .= "<strong style='font-size: 110%;'>$i</strong>";
                } else {
                    $output .= $i;
                }
                $output .= "</a>&nbsp;&nbsp;";
            }
            if ($_GET[$page_variable] != $pages) {
                $output .='&nbsp;<a href="' . $url . $page_variable . '=' . ($_GET[$page_variable] + 1) . '" title="Next page">&gt;</a>&nbsp;&nbsp;&nbsp;';
                $output .='<a href="' . $url . $page_variable . '=' . $pages . '" title="Last page ['.$pages.']">&gt;&gt;&gt;</a>';
            }
        }
        $output .= "</div>";
        return $output;
    }



      /**
       * This function processes a form and saves it to the given object
       * The format of $fields is identical to $fields in $this->form(), except for captcha fields
       * The object is merely populated, it is NOT saved
		 *
		 * If you have an recaptcha input type in your form, and it fails, then this function returns null
		 * Be sure to modify your field "name" for recaptcha to be your recaptcha private key
       */
      public function post(&$object, $fields = array()) {
         $basic = array("input_text", "input_textarea", "input_select", "input_checkbox", "input_radio", "input_state");
         $datetime = array("input_date_select", "input_time_select", "input_datetime_select", "input_calendar");
         foreach ($fields as $field) {
            if ($field["type"] == "file") {
               $object->set($field["name"], $this->upload_file($field["value"]["class"], $field["value"]["id"], $field["name"]));
            }
            else if ($field["type"] == "input_recaptcha") {
					if (!$this->post_recaptcha($field["name"])) {
						return null;
					}
            }
				else if (in_array($field["type"], $basic)) {
               $object->set($field["name"], $_POST[$field["name"]]);
            }
            else if (in_array($field["type"], $datetime)) {
               $object->set($field["name"], $this->post_datetime($field["name"]));
            }
         }
      }



      /**
       * Takes the base input name given to one of the datetime input helpers, and returns the time
       */
      public function post_datetime($input_name) {
         if (!empty($_POST[$input_name])) {
            return strtotime($_POST[$input_name]);
         }
         else {
            $year = empty($_POST[$input_name . "_year"]) ? 0 : $_POST[$input_name . "_year"];
            $month = empty($_POST[$input_name . "_month"]) ? 0 : $_POST[$input_name . "_month"];
            $day = empty($_POST[$input_name . "_day"]) ? 0 : $_POST[$input_name . "_day"];
            $hour = empty($_POST[$input_name . "_hour"]) ? 0 : $_POST[$input_name . "_hour"];
            $minute = empty($_POST[$input_name . "_minute"]) ? 0 : $_POST[$input_name . "_minute"];
            $second = empty($_POST[$input_name . "_second"]) ? 0 : $_POST[$input_name . "_second"];
            $ante = empty($_POST[$input_name . "_ante"]) ? 0 : $_POST[$input_name . "_ante"];
            if ($ante) {
               $hour += 12;
            }
            return strtotime("$year-$month-$day $hour:$minute:$second");
         }
      }



      /**
       * returns true if the captcha was passed, false if it was not
       */
      public function post_recaptcha($private_key) {
			$plugin_name = $this->plugin_folder;
         $site_dir = $this->site_dir();
         $file_dir = "$site_dir/wp-content/plugins/$plugin_name/classes/onWord_helper/recaptchalib.php";
         require_once($file_dir);
			$resp = recaptcha_check_answer ($private_key, $_SERVER["REMOTE_ADDR"], $_POST["recaptcha_challenge_field"], $_POST["recaptcha_response_field"]);
			return $resp->is_valid;
      }



      /**
       * Searches the metadata of all the pages for the one that handles this action
       * Returns the url if it finds such a page, returns an empty string otherwise
       * Works in an AJAX context!
       */
      public function route($class, $method) {
         $site_dir = $this->site_dir();
         if (!is_callable("get_permalink")) {
            require_once ("$site_dir/wp-blog-header.php");
            require_once ("$site_dir/wp-includes/link-template.php");
         }
         include ("$site_dir/wp-config.php");
         $query = "SELECT post_id FROM " . $table_prefix . "postmeta WHERE meta_key = 'onWord_route' AND meta_value = '$class:$method'";
         if (!$result = mysql_query($query)) {die("<p>Failed to determine route:</p>" . mysql_error());}
         if (mysql_num_rows($result) == 0) {return "";}
         $row = mysql_fetch_assoc($result);
         $id = $row["post_id"];
         return get_permalink($id);
      }



      /**
       * Returns the WordPress directory, without a trailing slash
       * Works in an AJAX context!
       */
      public function site_dir() {
         $current_dir = __DIR__;
         while ($current_dir != "/" && !file_exists($current_dir . "/wp-config.php")) {
            $current_dir = dirname($current_dir);
         }
         $path = ($current_dir != "/")
            ? str_replace("\\", "/", $current_dir)
            : "";
         return $path;
      }



      /**
       * Returns the wordpress base url, without a trailing slash
       * Works in an AJAX context!
       */
      public function site_url() {
         //load wordpress if need be
         if (!is_callable("get_site_url")) {
            $site_dir = $this->site_dir();
            require_once ("$site_dir/wp-blog-header.php");
            require_once ("$site_dir/wp-includes/link-template.php");
         }
         $url = get_site_url();
         if (substr($url, -1) == "/") {
            return substr($url, 0, -1);
         }
         return $url;
      }



      /**
       * This will help you with processing forms containing files
       *
       * $class and $id are important because they tell the function where in the wp-content/uploads directory to store the file
       *
       * returns the file name
       */
      public function upload_file($class, $id, $input_name) {
			$filename = "";
         if (!empty($_FILES[$input_name]["name"])) {
            //delete the file that's being replaced, if there is one
				if (!empty($_POST["existing_$input_name"])) {
					$this->delete_file($class, $id, $_POST["existing_$input_name"]);
				}
				//move the file
            $site_dir = $this->site_dir();
            $class_dir = "$site_dir/wp-content/uploads/$class";
            $file_dir = "$class_dir/$id";
            mkdir($class_dir);
            mkdir($file_dir);
				$source_file = $_FILES[$input_name]["tmp_name"];
				$dest_file = "$file_dir/" . $_FILES[$input_name]["name"];
				if (file_exists($dest_file) || move_uploaded_file($source_file, $dest_file)) {
					$filename = $_FILES[$input_name]["name"];
            }
         }
         else {
            $filename = $_POST["existing_$input_name"];
         }
         if ($_POST["remove_$input_name"] == "remove") {
            $this->delete_file($class, $id, $_POST["existing_$input_name"]);
            $filename = "";
         }
         return $filename;
      }
   }
