<?php
   class onWord_object {
      /**
       * An onWord object is a simple ORM class designed to be used with the
       * WordPress code generation components.  It handles the basics of saving and loading objects
       * Of note, is that it uses lazy loading and saving to and from the database,
       * so if a programmer only needs a single value, they don't have to load the whole record into memory.
       * Of course, if you want to do this, you can use the save/load methods rather than the get/set methods.
       *
       * This is why $keys is a static array of property names instead of using PHP properties, to accomodate the extra meta-data that should not be user-facing.
       *
       * Also, a value can be anything, including an array, and it will be JSON encoded before it's put into the database
       *
       * Since WordPress uses the mysql library instead of mysqli, we must use mysql.
       *
       */

      private $id;
      private $saved;
      private $loaded;
      private $values;
      private $deleted;
      public static $keys = array();

      /**
       * Pass this constructor the id of an existing object to designate it for that use,
       * or leave it empty to create a new object
       */
      public function __construct($id = 0) {
         if (!is_numeric($id)) {
            return false;
         }
         $this->id = $id;
         $this->saved = array();
         $this->loaded = array();
         $this->values = array();
         $class = get_class($this);
         $keys = $class::$keys;
         foreach ($keys as $key=>$type) {
            $this->saved[$key] = false;
            $this->loaded[$key] = false;
            $this->values[$key] = "";
         }
         $this->deleted = false;
      }


      private static function create_object() {
         $class = get_called_class();
         $query = "INSERT INTO $class (id) VALUES ('')";
         if (!mysql_query($query)) {die("<p>Object creation failed:</p>" . mysql_error());}
         $id = mysql_insert_id();
         return $id;
      }


      /**
       * Deletes this object from the database, and marks the RAM copy as deleted
       * Returns false if the object's id is invalid, or if the object is already deleted, true if the deletion succeeded
       */
      public function delete() {
         if (is_numeric($this->id) && $this->id > 0 && !$this->deleted) {
            $class = get_class($this);
            $query = "DELETE FROM $class Where id = " . $this->id;
            if (!mysql_query($query)) {die("<p>Failed to delete object:</p>" . mysql_error());}
            $this->deleted = true;
            return true;
         }
         return false;
      }


      /**
       * This function is for getting a single value.  If the value is already loaded, no query is performed
       * You may also use "id" for the key and get the object's current id
       * Or "deleted" to find out if the object is marked as deleted or not
       */
      public function get($key) {
         if ($key == "id") {return $this->id;}
         if ($key == "deleted") {return $this->deleted;}
         if ($this->loaded[$key]) {
            return $this->values[$key];
         }
         if (!is_numeric($this->id)) {
            return false;
         }
         $class = get_class($this);
         $query = "SELECT $key FROM $class WHERE id = " . $this->id;
         if (!$result = mysql_query($query)) {die("<p>Value retrieval failed:</p>" . mysql_error());}
         $row = mysql_fetch_assoc($result);
         $value = stripslashes($row[$key]);
			if (!is_numeric($value) && substr($value, 0, 5) == "JSON:") {
				$value = json_decode(substr($value, 5) , true);
			}
         $this->values[$key] = $value;
         $this->loaded[$key] = true;
         $this->saved[$key] = true;
         return $value;
      }


      /**
       * This sets up the table when the generated plugin is activated.
       *
       * Eventually, this should be generated by the application factory to be more "type" optimized.
       * Until then, you should probably override this function.
       *
       * All of the other functions will make properly formed queries regardless of the underlying data types, so you could even change the database schema after the plugin is installed if you don't like having every field be "text".
       *
       */
      public static function install() {
         $class = get_called_class();
         $keys = $class::$keys;
			if (!empty($keys)) {
				$query = "CREATE TABLE IF NOT EXISTS $class (
				id int(11) NOT NULL AUTO_INCREMENT,
				INDEX (id),
				PRIMARY KEY (id)
				)";
				if (!mysql_query($query)) {die ("<p>Failed to install class:</p>" . mysql_error());}
				foreach ($keys as $key=>$type) {
					$query = "ALTER TABLE $class ADD $key $type NOT NULL";
					mysql_query($query);
				}
			}
      }


      /**
       * Loads any unloaded data and caches it in the object using a single query
       * This function should be used to load all of the object, get() should be used to load individual values
       * Returns true if a query was performed
		 *
		 * $keys is an array of strings containing the names of the keys to load
       */
      public function load($load = false) {
         if (!is_numeric($this->id)) {
            return false;
         }
         $class = get_class($this);
         //figure out which values need to get loaded
			$loading = array();
			$keys = ($load === false)
				? array_keys($class::$keys)
				: $load;
			foreach ($keys as $key) {
				if (!$this->loaded[$key]) {
					$loading[] = $key;
				}
			}
         //load and parse all data values
         if (!empty($loading)) {
            $query = "SELECT " . implode(", ", $loading) . " FROM $class WHERE id = " . $this->id;
            if (!$result = mysql_query($query)) {die("<p>Failed to load object:</p>" . mysql_error());}
            $row = mysql_fetch_assoc($result);
            foreach($loading as $key) {
					$value = stripslashes($row[$key]);
					if (!is_numeric($value) && substr($value, 0, 5) == "JSON:") {
						$value = json_decode(substr($value, 5) , true);
					}
               $this->values[$key] = $value;
               $this->loaded[$key] = true;
               $this->saved[$key] = true;
            }
            return true;
         }
         return false;
      }


      /**
       * Updates any unsaved data
       * This function should be used to save a whole object, set() with update=true should be used to update individual values
       * returns true if a query was performed
       */
      public function save() {
         if (!is_numeric($this->id)) {
            return false;
         }
         $class = get_class($this);
         //get an id for this object if need be
         if ($this->id == 0) {
            $this->id = $class::create_object();
         }
         //get the list of unsaved keys
         $saving = array();
         $keys = $class::$keys;
         foreach ($keys as $key=>$type) {
            if (!$this->saved[$key] && $this->loaded[$key]) {
               //encode the values
               $value = $this->values[$key];
					if (!is_numeric($value)) {
						if (!is_string($value)) {
							$value = "JSON:" . json_encode($value);
						}
						$value = "'" . mysql_real_escape_string($value) . "'";
					}
               $saving[] = "$key = $value";
               $this->saved[$key] = true;
               $this->loaded[$key] = true;
            }
         }
         if (!empty($saving)) {
            //we have something to update, do it
            $query = "UPDATE $class SET ";
            $query .= implode(", ", $saving) . " WHERE id = " . $this->id;
            if (!mysql_query($query)) {die("<p>Failed to save object:</p>" . mysql_error());}
            return true;
         }
         return false;
      }


      /**
       * This function is for setting a single value
       * To perform an immediate query to update the value in the databse without using save, pass true to the update parameter
       * Returns true if a query was performed
       */
      public function set($key, $value, $update = false) {
         //set the value
         $this->values[$key] = $value;
         $this->saved[$key] = false;
			$this->loaded[$key] = true;
         //see if this value needs to be saved
         if ($update) {
            //get an id for this object if need be
            if (!is_numeric($this->id)) {
               return false;
            }
            $class = get_class($this);
            //get an id for this object if need be
            if ($this->id == 0) {
               $this->id = $class::create_object();
            }
            if (!is_numeric($value)) {
					if (!is_string($value)) {
						$value = "JSON:" . json_encode($value);
					}
					$value = "'" . mysql_real_escape_string($value) . "'";
				}
            $query = "UPDATE $class SET $key = $value WHERE id = " . $this->id;
            if (!mysql_query($query)) {die("<p>Value update failed:</p>" . mysql_error());}
            $this->saved[$key] = true;
            $this->loaded[$key] = true;
         }
         return $update;
      }
   }
