<?php
   class onWord_query {
      public $arguments;
      public $results;
      public $error;
      public $num_rows;

      /**
       * $arguments is an array with the following keys: class, source, search_type, key, value, operator, sort, sort_direction, page, results_per_page
       *
       * class is a string, and determines which class of objects (the table) we're searching
       *
       * source is an array of object id's
       *
       * search_type can be "within" to search those id's within source, or it can be "combine" to perform a new search and append the results to source while removing duplicate keys
       *
       * key is a string, and holds the data_key to search for
       *
       * value is a string, and holds the data_value to search for
       *
       * operator is a string holding the comparison operator with values such as "=", ">", or "LIKE"
       * operator can also be "IN" in which case value can be an array and will be handled appropriately
       *
       * sort is the name of a key to sort the results by
       *
       * sort_direction can be "ASC" or "DESC" and will modify the direction of the sort to ascending and descending, respectively
       *
       * page is an integer that directs automatic pagination, or zero to disable pagination
       *
       * results_per_page is an integer that controls the page size for automatic pagination, and is ignored if page is zero
       */
      public function __construct($arguments = "") {
         $this->arguments = (is_array($arguments)) ? $arguments : array();
         $this->results = array();
         $this->error = "";
         $this->num_rows = 0;
      }



      /**
       * This function uses only the class, and source keys of the argument array
       * All records identified by the list of id's given by source, in the class given by class, will be deleted
       */
      public function delete() {
         $class = $this->arguments["class"];
         $source = (empty($this->arguments["source"]) || !is_array($this->arguments["source"])) ? array() : $this->arguments["source"];
         $query = "DELETE FROM $class WHERE id IN (" . implode(", ", $source) . ")";
         if (!mysql_query($query)) {
            $this->error = mysql_error();
            return false;
         }
         $this->results = array();
         $this->num_rows = 0;
         $this->error = "";
         return true;
      }



		/**
		 * Runs the query given by $query, and returns an array of rows while
		 * setting the member variables
		 */
		public function query($query) {
         //run the query
         if (!$result = mysql_query($query)) {
				$this->error = mysql_error();
            return false;
         }
         $this->num_rows = mysql_num_rows($result);
         $this->error = "";
         $this->results = array();
			while ($row = mysql_fetch_assoc($result)) {
				$this->results[] = $row;
			}
			return $this->results;
		}


      /**
       * Runs the search query considering all arguments stored in $this->arguments, and returns the resulting array of object id's
       *
       * After this function is executed, the query object's results property will hold the resulting array of id's, or an empty array.  The error property will hold any mysql_error's generated, and num_rows will hold the value of mysql_num_rows($result)
       */
      public function search() {
         //prepare the arguments
         $class = $this->arguments["class"];
         $source = (empty($this->arguments["source"]) || !is_array($this->arguments["source"])) ? array() : $this->arguments["source"];
         $search_type = (empty($this->arguments["search_type"])) ? "combine" : $this->arguments["search_type"];
         $key = (empty($this->arguments["key"])) ? "id" : $this->arguments["key"];
         $value = (empty($this->arguments["value"])) ? "0" : $this->arguments["value"];
         $operator = (empty($this->arguments["operator"])) ? ">" : mysql_real_escape_string($this->arguments["operator"]);
         $sort = (empty($this->arguments["sort"])) ? "id" : $this->arguments["sort"];
         $sort_direction = (empty($this->arguments["sort_direction"])) ? "ASC" : $this->arguments["sort_direction"];
         $page = (empty($this->arguments["page"])) ? 0 : $this->arguments["page"];
         $results_per_page = (empty($this->arguments["results_per_page"])) ? 30 : $this->arguments["results_per_page"];
         //start the query
         $query = "SELECT DISTINCT id FROM $class";

         //add the key
         $query .= " WHERE $key ";

         //add the value, being sure to treat numbers like numbers and arrays like arrays
         if (!is_numeric($value)) {
            if (is_array($value)) {
               foreach ($value as $index => $value_at) {
                  if (!is_numeric($value_at)) {
                     $value[$index] = "'" . mysql_real_escape_string($value_at) . "'";
                  }
               }
               $value = "(" . implode(", ", $value) . ")";
            }
            else {
               $value = "'" . mysql_real_escape_string($value) ."'";
            }
         }
         $query .= " $operator $value";

         //add the search domain
         if ($search_type == "within") {
            if (!empty($source)) {$query .= " AND id IN (" . implode(", ", $source) . ")";}
            else {$query .= " AND id <> id";}
         }

         //add the sort
         if ($sort) {
            $query .= " ORDER BY $sort $sort_direction";
         }

         //add the limit
         if ($page != 0) {
            $offset = ($page - 1) * $results_per_page;
            $query .= " LIMIT $offset, $results_per_page";
         }

         //run the query
         if (!$result = mysql_query($query)) {
				$this->error = mysql_error();
            return false;
         }
         $this->num_rows = mysql_num_rows($result);
         $this->error = "";
         $this->results = ($search_type == "within") ? array() : $source;

         //compile the results
         while ($row = mysql_fetch_assoc($result)) {
             if (!in_array($row["id"], $this->results)) {$this->results[] = $row["id"];}
         }
         return $this->results;
      }



      /**
       * This function uses only the class, key, value, and source keys of the arguments array
       * The source array determines what objects of the given class will have their values updated
       * key and value designate what value will be set for which key
       */
      public function update() {
         $class = $this->arguments["class"];
         $source = (empty($this->arguments["source"]) || !is_array($this->arguments["source"])) ? array() : $this->arguments["source"];
         $key = (empty($this->arguments["key"])) ? "" : $this->arguments["key"];
         $value = (empty($this->arguments["value"])) ? "" : mysql_real_escape_string(json_encode($this->arguments["value"]));
         if (!is_numeric($value)) {$value = "'$value'";}
         $query = "UPDATE $class SET $key = $value WHERE id IN (" . implode(", ", $source) . ")";
         if (!mysql_query($query)) {
            $this->error = mysql_error();
            return false;
         }
         $this->results = array();
         $this->num_rows = 0;
         $this->error = "";
         return true;
      }
   }